@@ -1,14 +1,15 @@
-import sys, os
-import platform
+import numpy
 from PyQt5.QtWidgets import QApplication
 
 from orangewidget import gui
 from orangewidget.settings import Setting
 from oasys.widgets import gui as oasysgui, congruence
 
-from orangecontrib.xoppy.util.xoppy_util import locations
 from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget
 
+from xoppylib.xoppy_run_binaries import xoppy_calc_xtube_w
+from oasys.widgets.exchange import DataExchangeObject
+
 class OWxtube_w(XoppyWidget):
     name = "Tube_W"
     id = "orange.widgets.dataxtube_w"
@@ -22,6 +23,8 @@ class OWxtube_w(XoppyWidget):
     RIPPLE = Setting(0.0)
     AL_FILTER = Setting(0.0)
 
+    def __init__(self):
+        super().__init__(show_script_tab=True)
 
     def build_gui(self):
 
@@ -60,7 +63,6 @@ class OWxtube_w(XoppyWidget):
     def unitFlags(self):
          return ['True','True','True']
 
-
     def get_help_name(self):
         return 'xtube_w'
 
@@ -71,7 +73,72 @@ class OWxtube_w(XoppyWidget):
         self.AL_FILTER = congruence.checkPositiveNumber(self.AL_FILTER, "Al filter")
 
     def do_xoppy_calculation(self):
-        return xoppy_calc_xtube_w(VOLTAGE=self.VOLTAGE,RIPPLE=self.RIPPLE,AL_FILTER=self.AL_FILTER)
+        out_file = xoppy_calc_xtube_w(VOLTAGE=self.VOLTAGE,RIPPLE=self.RIPPLE,AL_FILTER=self.AL_FILTER)
+        dict_parameters = {
+                        "VOLTAGE"   : self.VOLTAGE,
+                        "RIPPLE"    : self.RIPPLE,
+                        "AL_FILTER" : self.AL_FILTER,
+                        }
+
+        script = self.script_template().format_map(dict_parameters)
+
+        self.xoppy_script.set_code(script)
+        return out_file, script
+
+    def script_template(self):
+        return """
+#
+# script to make the calculations (created by XOPPY:xtube_w)
+#
+import numpy
+import scipy.constants as codata
+from xoppylib.xoppy_run_binaries import xoppy_calc_xtube_w
+
+out_file =  xoppy_calc_xtube_w(
+        VOLTAGE = {VOLTAGE},
+        RIPPLE    = {RIPPLE},
+        AL_FILTER = {AL_FILTER},
+        )
+
+# data to pass to power
+data = numpy.loadtxt(out_file)
+energy = data[:,0]
+flux = data[:,1] # photons/1keV(bw)/mA/mm^2(@1m)/s
+spectral_power = flux * 1e3 * energy * codata.e # W/eV/mA/mm^2(@1m)
+cumulated_power = spectral_power.cumsum() * numpy.abs(energy[1]-energy[0]) # W/mA/mm^2(@1m)
+
+#
+# example plot
+#
+from srxraylib.plot.gol import plot
+plot(energy,flux,
+    xtitle="Photon energy [eV]",ytitle="Flux [photons/1keV(bw)/mA/mm^2(@1m)/s]",title="xtube_w Flux",
+    xlog=False,ylog=False,show=False)
+plot(energy,spectral_power,
+    xtitle="Photon energy [eV]",ytitle="Spectral Power [W/eV/mA/mm^2(@1m)]",title="xtube_w Spectral Power",
+    xlog=False,ylog=False,show=False)
+plot(energy,cumulated_power,
+    xtitle="Photon energy [eV]",ytitle="Cumylated Power [W/mA/mm^2(@1m)]",title="xtube_w Cumulated Power",
+    xlog=False,ylog=False,show=True)
+    
+#
+# end script
+#
+"""
+
+
+    def extract_data_from_xoppy_output(self, calculation_output):
+
+        spec_file_name, script = calculation_output
+        out = numpy.loadtxt(spec_file_name)
+        if len(out) == 0 : raise Exception("Calculation gave no results (empty data)")
+
+        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
+        calculated_data.add_content("xoppy_specfile", spec_file_name)
+        calculated_data.add_content("xoppy_data", out)
+        calculated_data.add_content("xoppy_script", script)
+
+        return calculated_data
 
     def get_data_exchange_widget_name(self):
         return "XTUBE_W"
@@ -85,39 +152,11 @@ class OWxtube_w(XoppyWidget):
     def getYTitles(self):
         return ["Flux [photons/1keV(bw)/mA/mm^2(@1m)/s])"]
 
-# --------------------------------------------------------------------------------------------
-# --------------------------------------------------------------------------------------------
 
-def xoppy_calc_xtube_w(VOLTAGE=100.0,RIPPLE=0.0,AL_FILTER=0.0):
-    print("Inside xoppy_calc_xtube_w. ")
-
-    for file in ["tasmip_tmp.dat"]:
-        try:
-            os.remove(os.path.join(locations.home_bin_run(),file))
-        except:
-            pass
-
-    try:
-        with open("xoppy.inp","wt") as f:
-            f.write("%f\n%f\n%f\n"%(VOLTAGE,RIPPLE,AL_FILTER))
-
-
-        if platform.system() == "Windows":
-            command = "\"" + os.path.join(locations.home_bin(),'tasmip.exe\" < xoppy.inp')
-        else:
-            command = "'" + os.path.join(locations.home_bin(), 'tasmip') + "' < xoppy.inp"
-
-        print("Running command '%s' in directory: %s \n"%(command,locations.home_bin_run()))
-        print("\n--------------------------------------------------------\n")
-        os.system(command)
-        print("\n--------------------------------------------------------\n")
-
-        return "tasmip_tmp.dat"
-    except Exception as e:
-        raise e
 
 
 if __name__ == "__main__":
+    import sys
     app = QApplication(sys.argv)
     w = OWxtube_w()
     w.show()
